@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use v5.10;
 
-our $VERSION = '0.23';
+our $VERSION = '0.25';
 
 use RDF::aREF::Query;
 use RDF::aREF::Decoder;
@@ -15,6 +15,8 @@ use parent 'Exporter';
 our @EXPORT = qw(decode_aref encode_aref aref_query aref_query_map);
 our %EXPORT_TAGS = (all => [@EXPORT]);
 
+our @CARP_NOT = qw(RDF::aREF::Query RDF::aREF::Decoder RDF::aREF::Encoder);
+
 sub decode_aref(@) { ## no critic
     my ($aref, %options) = @_;
     RDF::aREF::Decoder->new(%options)->decode($aref);
@@ -33,31 +35,33 @@ sub encode_aref(@) { ## no critic
         $encoder->add_iterator( $source->as_stream, $aref );
     } elsif (blessed $source and $source->DOES('Attean::API::TripleStore')) {
         $encoder->add_iterator( $source->get_triples, $aref );
-    } elsif (!ref $source and $source =~ qr{^https?://}) {
-       if (eval { require RDF::Trine::Model; require RDF::Trine::Parser; 1 }) {
-            my $model = RDF::Trine::Model->new;
-            RDF::Trine::Parser->parse_url_into_model($source, $model); # TODO: use iterator
-            $encoder->add_iterator( $model->as_stream, $aref );
-       } else {
-           croak "encoding aREF from URL not supported. Install RDF::Trine or Attean!";
-       }
     } elsif (ref $source and reftype $source eq 'HASH') {
         $encoder->add_hashref( $source, $aref );
-    } # TODO: add via callback with code reference?
+    } elsif (!ref $source) {
+        eval { require RDF::Trine::Model; require RDF::Trine::Parser };
+        croak "RDF::Trine missing: encoding aREF from URL or file not supported!" if $@;
+        my $model = RDF::Trine::Model->new;
+        # TODO: directly use iterator
+        if ($source =~ qr{^https?://}) {
+            RDF::Trine::Parser->parse_url_into_model($source, $model); 
+        } elsif (-f $source) {
+            my $parser = RDF::Trine::Parser->guess_parser_by_filename($source);
+            $parser->parse_file_into_model("file://$source", $source, $model)
+        } else {
+            croak 'invalid RDF graph, given as string';
+        }
+        $encoder->add_iterator( $model->as_stream, $aref );
+    }
     
     return $aref;
 }
 
 sub aref_query(@) { ## no critic
     my ($graph, $origin, @queries) = @_ < 3 ? ($_[0], undef, $_[1]) : @_;
-
-    map { $_->apply($graph, $origin) }
-    map { (blessed $_ and $_->isa('RDF::aREF::Query')) 
-          ? $_ : RDF::aREF::Query->new( query => $_ ) } @queries;
+    RDF::aREF::Query->new( query => join '|', @queries )->apply($graph, $origin);
 }
 
-
-sub aref_query_map {
+sub aref_query_map(@) { ## no critic
     my ($graph, $origin, $map) = @_ < 3 ? ($_[0], undef, $_[1]) : @_;
 
     my %record;
@@ -154,8 +158,8 @@ Equivalent to C<< RDF::aREF::Decoder->new(%options)->decode($aref) >>.
 
 =head2 encode_aref $graph [, %options ]
 
-Construct an aREF subject map (using a new L<RDF::aREF::Encoder>) fron an RDF
-graph. The graph can be supplied as:
+Construct an aREF subject mapfrom an RDF graph. The L<RDF::aREF::Encoder> for
+possible options. The C<$graph> can be supplied as:
 
 =over
 
@@ -169,7 +173,7 @@ instance of L<RDF::Trine::Model::Iterator>
 
 =item 
 
-an URL (experimental and only if RDF::Trine is installed)
+an URL or a filename (only if L<RDF::Trine> is installed)
 
 =item
 
